/* src/chanserv.c - Diverses commandes sur le module chanserv
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Gay@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

CVSINFO("$Id: chanserv.c,v 1.56 2003/08/04 20:55:37 romexzf Exp $");

extern const char *pasdeperm;

/*
 * show_access
 *
 * parv[0] = bot
 * parv[1] = chan
 * parv[2] = recherche
 */
int show_access(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	register aLink *lp;
	char *nick2 = parv[2], comp = '\0';
	int i = 0, autoop, protect, suspend, autovoice, l = 0, wild = 0;

	if(*nick2 == '>' || *nick2 == '<' || *nick2 == '=')
	{
		comp = *nick2++;
		if(!*nick2 || !is_num(nick2))
			return csntc(nick, "La valeur du niveau doit tre numrique derrire <,> ou =");
		l = atoi(nick2);
	}
	else wild = (strchr(nick2, '*') || strchr(nick2, '?'));
	autoop = getoption("-autoop", parv, parc, 2, -1);
	autovoice = getoption("-autovoice", parv, parc, 2, -1);
	protect = getoption("-protect", parv, parc, 2, -1);
	suspend = getoption("-suspend", parv, parc, 2, -1);
	csntc(nick, "------------------[ \0032\2%s\2\0036 > Liste des accs\3      ]------------------", chaninfo->chan);

	for(lp = chaninfo->access;lp;lp=lp->next)
	{
		if(!AWait(lp->value.a)	&& (
			 (comp != '\0' && ((comp == '>' && lp->value.a->level > l) /* recherche */
			                 || (comp == '<' && lp->value.a->level < l)	/* via level */
			                 || (comp == '=' && l == lp->value.a->level)))
			  || (*nick2 == '*' && !*(nick2+1)) /* recherche sur '*' */
			  || (wild /* ou sur un vrai mask */
			  		&& !match(nick2, lp->value.a->user->nick))
			  || !strcasecmp(nick2, lp->value.a->user->nick)) /* ou access exact */
			&& (!autoop || AOp(lp->value.a)) /* ET match des options Aop/Avoice/Suspend etc */
			&& (!autovoice || AVoice(lp->value.a))
			&& (!protect || AProtect(lp->value.a))
			&& (!suspend || ASuspend(lp->value.a)))
		{
			show_accessn(lp->value.a, lp->value.a->user, nick);
			csntc(nick, "-");
			i++;
		}
	}
	return csntc(nick, "Un Total de %d entre%s trouve%s", i, PLUR(i), PLUR(i));
}

void show_accessn(anAccess *acces, anUser *user, aNick *num)
{
	csntc(num, "\2Username:\2 %s \2Niveau:\2 %d \2Identifi:\2 %s", user->nick, acces->level, user->n ? "Oui" : "Non");
	csntc(num, "\2Options:\2%s", GetAccessOptions(acces));
	csntc(num, "\2Dernire visite:\2 %s", acces->lastseen == 1 ? "Actuellement dans le salon" : get_time(acces->lastseen));

	if(acces->info) csntc(num, "\2Infoline:\2 %s", acces->info);
	return;
}

/*
 * add_user
 *
 * parv[1] = salon
 * parv[2] = username
 * parv[3] = niveau
 */
int add_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	char *username = parv[2], *salon = parv[1]
#ifdef USE_MEMOSERV
,memo[MEMOLEN + 1]
#endif
;
	int niveau;

	if(!(u = ParseNickOrUser(nick, username))) return 0;

	if(!is_num(parv[3]) || (niveau = atoi(parv[3])) < 1 || niveau >= OWNERLEVEL)
		return csntc(nick, "Le niveau doit tre un nombre entre 1 et %d", OWNERLEVEL - 1);

	if(!nick->user || (niveau >= ChanLevelbyUserI(nick->user, salon) && !IsAnAdmin(nick->user)))
		return csntc(nick, "Le niveau ne doit pas tre plus grand que le votre.");

	if(u->flag & U_PREJECT)
		return csntc(nick, "\2%s\2 refuse systmatiquement les propositions d'accs.", u->nick);

	else if(u->flag & U_PACCEPT)
	{
		for(a=u->accesshead;a && strcasecmp(a->chan, salon);a=a->next);
		if(a) return csntc(nick, "%s a dj %saccs sur %s.",
						username, AWait(a) ? "une proposition d'" : "un ", salon);

		add_access(u, salon, niveau, (niveau >= 400) ? (A_OP | A_PROTECT) : 0, time(NULL));
		write_anuser(u);
		csntc(nick, "\2%s\2 a maintenant un accs sur \2%s\2 (%d)", u->nick, salon, niveau);
		if(u->n) csntc(u->n, "\2%s\2 vous a donn un accs sur \2%s\2 (%d)", nick->nick, salon, niveau);
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, "\2%s\2 vous a donn un accs sur \2%s\2 (%d)", nick->nick, salon, niveau);
			add_memo(u, nick->user->nick, time(NULL), memo, !!u->n);
		}
#endif
		return 1;
	}
	else
	{
		for(a=u->accesshead;a && strcasecmp(a->chan, salon);a=a->next);
		if(a)
		{
			if(AWait(a)) csntc(nick, "Vous avez dj propos un access  %s.", username);
			else csntc(nick, "%s a dj un accs sur %s.", username, salon);
			return 1;
		}

		add_access(u, salon, niveau, (niveau >= 400) ? (A_OP | A_PROTECT | A_WAITACCESS) : A_WAITACCESS, time(NULL));
		csntc(nick, "Vous avez bien propos  %s un access sur %s. Il va recevoir une notice ou "
			"un memo lui demandant de confirmer ou de refuser cet access", username, salon);

		if(u->n)
		{
			csntc(u->n, "%s vous a propos un access de %d sur %s.", nick->nick, niveau, salon);
			csntc(u->n, "Pour accepter tapez:\2 /%s %s ACCEPT %s \2, pour refuser tapez:\2 /%s %s REFUSE %s",
				cs.nick, RealCmd("myaccess"), salon, cs.nick, RealCmd("myaccess"), salon);
		}
#ifdef USE_MEMOSERV
		else
		{
			snprintf(memo, MEMOLEN, "%s vous a propos un access de %d sur %s. Pour accepter "
				"tapez:\2 /%s %s ACCEPT %s \2, pour refuser tapez:\2 /%s %s REFUSE %s",
			nick->nick, niveau, salon, cs.nick, RealCmd("myaccess"), salon, cs.nick, RealCmd("myaccess"), salon);
			add_memo(u, nick->user->nick, time(NULL), memo, !!u->n);
		}
#endif
		write_anuser(u);
		return 1;
	}
	return 0; /* on sait jamais :P */
}

/*
 * del_user
 *
 * parv[1] = #salon
 * parv[2] = username
 */
int del_user(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	anUser *u;
	anAccess *a;
	const char *salon = parv[1];

	if(!(u = ParseNickOrUser(nick, parv[2]))) return 0;

	if(!(a = GetAccessIbyUserI(u, salon)))
		return csntc(nick, "%s n'a pas d'accs sur %s", u->nick, salon);

	if(ChanLevelbyUserI(nick->user, salon) <= a->level && !IsAnAdmin(nick->user))
		return csntc(nick, "Vous ne pouvez pas deluser un user d'un niveau suprieur ou gal au votre.");

	if(a->level == OWNERLEVEL)
		return csntc(nick, "Vous ne pouvez pas deluser l'owner du salon. "
			"Utilisez CHAN DEL pour supprimer le salon ou CHAN CHOWNER pour changer de propritaire.");

	del_access(u, chaninfo);
	write_anuser(u);

	csntc(nick, "%s n'a plus d'accs sur %s", u->nick, salon);
	if(u->n) csntc(u->n, "%s a supprim votre accs sur %s.", nick->nick, salon);
	return 1;
}

int kick(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	const char *chan = parv[1], *cible = parv[2];
	aNick *targ;

	if(!strchr(cible, '*') && !strchr(cible, '?'))
	{	/*la cible est un nick unique .. */

		if(!strcasecmp(cs.nick, cible))
			return csntc(nick, "Je suis pas sur que a soit prfrable pour \2%s\2...", chaninfo->chan);

		if(!(targ = GetMemberIbyNick(chaninfo, cible)))
			return csntc(nick, "\2%s\2 n'est pas sur %s", cible, chaninfo->chan);

		if((targ->flag & N_SERVICE))
			return csntc(nick, "\2%s\2 est un Service. (+k)", targ->nick);

		if(!check_protect(nick, targ, chan)) return 0;

		cskick(chan, targ->numeric, "(\2%s\2) %s", nick->user->nick,
			(parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150));
	}
	else
	{	/* kick sur un mask*/
		char host[NUHLEN];
		char *r = (parc < 3) ? DEFRAISON : parv2msg(parc, parv, 3, 150);
		aLink *lp = chaninfo->members;

		for(;lp;lp=lp->next)
		{
			targ = lp->value.j->nick;
			snprintf(host, NUHLEN, "%s!%s@%s", targ->nick, targ->ident, targ->host);

			if(!match(cible, host) && check_protect(nick, targ, chan) && !(targ->flag & N_SERVICE))
				cskick(chan, targ->numeric, "(\2%s\2) %s", nick->user->nick, r);
		}
	}
	return 1;
}


int verify(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;

	if(!strcasecmp(parv[1], cs.nick)) return csntc(nick, "Yeah, c'est bien moi :)");

	if(!strcasecmp(nick->nick, parv[1])) n = nick;
	else n = getnickbynick(parv[1]);

	if(!n) return csntc(nick, NOTCONNECTED, parv[1]);

	if(n->flag & N_SERVICE) return csntc(nick, "%s est un Robot (\2Service\2).", n->nick);

	if(!n->user) return csntc(nick, "%s n'est pas actuellement logu.", n->nick);

	if(IsAnAdmin(nick->user) || n == nick)
		csntc(nick, "%s!%s@%s est logu sous l'username %s %s %s", n->nick, n->ident, n->host,
			n->user->nick, (n->user->level > 1) ? "- Administrateur des Services" : "", (n->flag & N_OPER) ? "- IRCop" : "");
	else csntc(nick, "%s est logu sous l'username %s %s %s", n->nick, n->user->nick,
			IsAnAdmin(n->user) ? "- Administrateur des Services" : "", (n->flag & N_OPER) ? " - IRCop" : "");
	return 1;
}

int mode(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	int a = 0;
	char *modes = parv[2];
	anAccess *acces;

	if(strchr(modes, 'o') || strchr(modes, 'b') || strchr(modes, 'v'))
		return csntc(nick, "Vous n'tes pas autoris  changer les Ops,Voices,Bans avec cette commande.");

	if(!(nick->flag & N_OPER) && strchr(modes, 'O'))
		return csntc(nick, "Vous n'tes pas autoris  changer ce mode (O)");

	if(chaninfo->cml && !IsAnAdmin(nick->user) && (acces = GetAccessIbyUserI(nick->user, chaninfo->chan)) && acces->level < chaninfo->cml)
		return csntc(nick, "Le ChModesLevel est  %d.", chaninfo->cml);

	if(strchr(modes, 'k')) a++;
	if(strchr(modes, 'l')) a++;

	csmode(chaninfo, "%s %s %s", modes, (a && parc > 2) ? parv[3] : "", (a > 1 && parc > 3) ? parv[4] : "");
	write_achan(chaninfo);

	return 1;
}

int topic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, "Le LOCKTOPIC est actif sur %s", parv[1]);

	cstopic(chaninfo, parv2msg(parc, parv, 2, TOPICLEN));
	write_achan(chaninfo);
	return 1;
}

int rdeftopic(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	if(CLockTopic(chaninfo)) return csntc(nick, "Le LOCKTOPIC est actif sur %s", parv[1]);

	if(chaninfo->deftopic[0] != '\0') cstopic(chaninfo, chaninfo->deftopic);
	return 1;
}

int rdefmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, "-%s %s", GetCModes(chaninfo), (chaninfo->mode & C_MKEY) ? chaninfo->key : "");
	if(chaninfo->defmodes[0] != '\0') csmode(chaninfo, "%s", chaninfo->defmodes);
	return 1;
}

int info(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aLink *lp = chaninfo->access;
	anAccess *access = NULL;

	for(;lp && lp->value.a->user != nick->user;lp=lp->next);
	if(!lp) return 0;
	access = lp->value.a;

	if(parc < 2)
		return csntc(nick, "Veuillez prciser une infoline ou le mot cl 'none' pour la supprimer.");

	if(!strcasecmp(parv[2], "none"))
	{
		if(access->info)
		{
			free(access->info);
			access->info = NULL;
		}
		csntc(nick, "Infoline sur \2%s\2 supprime.", access->chan);
	}
	else
	{
		str_dup(&access->info, parv2msg(parc, parv, 2, 199));
		csntc(nick, "Infoline sur \2%s\2 change en: %s", access->chan, access->info);
	}
	write_anuser(nick->user);
	return 1;
}

int invite(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	aNick *n;
	aJoin *j;

	if(parc < 2) n = nick;
	else if (!(n = getnickbynick(parv[2]))) return csntc(nick, NOTCONNECTED, parv[2]);

	if((j = getjoininfo(n, chaninfo->chan)))
		return csntc(nick, "%s %s dj sur %s.", (n != nick) ? n->nick : "Vous",
					(n != nick) ? "est" : "tes", chaninfo->chan);

	if(is_ban(n, chaninfo))
	{
		if(n == nick) csntc(nick, "Vous tes banni du salon %s : vous ne pouvez pas vous y inviter.", chaninfo->chan);
		else csntc(nick, "%s est banni du salon %s : vous ne pouvez l'y inviter", n->nick, chaninfo->chan);
		return 0;
	}

	putserv("%s " TOKEN_INVITE " %s :%s\r\n", cs.num, n->nick, chaninfo->chan);
	putserv("%s " TOKEN_WALLCHOPS " %s :Invitation de %s dans le salon %s par %s\r\n",
		cs.num, chaninfo->chan, n->nick, chaninfo->chan, nick->nick);
	return 1;
}

int clearmodes(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	csmode(chaninfo, "-%s %s", GetCModes(chaninfo), (chaninfo->mode & C_MKEY) ? chaninfo->key : "");
	csntc(nick, "Tous les modes de %s sont supprims.", parv[1]);
	return 1;
}

/*
 * see_alist
 *
 * parv[1] = chan
 */
int see_alist(aNick *nick, aChan *chan, int parc, char **parv)
{
	aLink *lp;

	if(!chan->access) return csntc(nick, "Personne n'est actuellement identifi pour %s.", chan->chan);

	csntc(nick, "------------------[ \0032\2%s\2\0036 > Liste des identifis\3 ]------------------", chan->chan);
	csntc(nick, "\2Pseudo Actuel Username Niveau");

	for(lp=chan->access;lp;lp=lp->next)
		if(lp->value.a->user->n)
			csntc(nick, "%-13s %-8s %-6d", lp->value.a->user->n->nick, lp->value.a->user->nick, lp->value.a->level);
	return 1;
}

int admin_say(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :%s\r\n", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

int admin_do(aNick *nick, aChan *chaninfo, int parc, char **parv)
{
	putserv("%s " TOKEN_PRIVMSG " %s :\001ACTION %s\001\r\n", cs.num, parv[1], parv2msg(parc, parv, 2, 400));
	return 1;
}

