/* src/del_info.c - Suppression d'informations en mmoire
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "main.h"

CVSINFO("$Id: del_info.c,v 1.44 2003/07/28 18:23:04 romexzf Exp $");

extern struct badchans *badchead;
extern aKill *killhead;
extern struct badnicks *badnhead;
extern struct ignore *ignorehead;
#ifdef HAVE_TRACK
extern struct track *trackhead;
#endif

void del_nickinfo(aNick *nick)
{
	aJoin *join, *chanp = NULL;
	int needwrite = 0;
	const time_t current = time(NULL);

	if(nick->user)	/* sauvergarde du lastlogin */
	{
#ifdef HAVE_TRACK
		struct track *track = trackhead, *tt = NULL;
		for(;track;track = tt)
		{
			tt = track->next;
			if(nick->user == track->tracked)
				csntc(track->tracker, "[\2Track\2] %s (User: %s) vient d'tre Kill.", nick->nick, track->tracked->nick);
			if(nick == track->tracker) del_track(track->tracked);
		}
#endif
		nick->user->lastlogin = realloc(nick->user->lastlogin,
			(strlen(nick->nick) + strlen(nick->ident) + strlen(nick->host) + 3));
		sprintf(nick->user->lastlogin, "%s!%s@%s", nick->nick, nick->ident, nick->host);

		nick->user->n = NULL;
		needwrite = 1;
	}
	del_killinfo(nick, NULL); /* purge de tous ses kills*/

	join = nick->joinhead;
	while(join)
	{
		chanp = join;
		if(join->chan) /* c'est un chan reg (conomie d'un getchaninfo..)*/
		{
			register aLink *lp;
			if(nick->user)
				for(lp=join->chan->access;lp;lp=lp->next)
					if(lp->value.a->user == nick->user)
					{ /* mise  jour du lastseen */
						lp->value.a->lastseen = current;
						needwrite = 1;/* on n'crit pas  chaque fois*/
						break;
					}

			del_link(join->chan, nick);
		}
		join = join->next;
		free(chanp);
	}
	if(needwrite && nick->user) write_anuser(nick->user);

	if(nick == nickhead)
	{
		nickhead = nick->next;
		if(nick->next) nickhead->last = NULL;
	}
	else
	{
		nick->last->next = nick->next;/*on change les positions en 'sautant' l'element a del*/
		if(nick->next) nick->next->last = nick->last;
	}

	free(nick);
	nick = NULL;
}

void del_abadchan(const char *chaine)
{
	struct badchans *current = badchead, *last = NULL;

	for(;current; last = current, current = current->next)
		if(!strcasecmp(chaine, current->chaine))
		{
			if(last) last->next = current->next;
			else badchead = current->next;
			free(current);
			return;
		}
}

void del_chan(aChan *chan, const char *raison)
{
	aChan *current = chanhead, *previous = NULL;
	char thechan[200], fichier[230];

	conv_char(chan->chan, thechan);
	snprintf(fichier, 230, DBDIR "/chans/%s", thechan);
	remove(fichier);
	snprintf(fichier, 230, DBDIR "/bans/%s", thechan);
	remove(fichier);
	/* Suppression de tous les accs du salon
	 (c chiant a demande d'aller voir tous les users :(
	 >> plus maintenant grace aux links (voir plus bas). On doit
	 cependant pour le moment supprimer tous les waitaccess en cherchant
	 tous les usernames vu qu'on a pas encore mis de lien entre chan et les
	 waitaccess
	 >> plus maintenant grce au flag WAIT_ACCESS*/

	for(;current && current != chan; previous = current, current = current->next);

	if(current)
	{
		aBan *ban, *bt;
		aLink *lp, *lpp;
		if(!previous) chanhead = current->next;
		else previous->next = current->next;
		/*nettoyage*/
		for(ban = current->banhead;ban;ban = bt)
		{
			bt = ban->next;
			free(ban->raison);
			free(ban);
		}
		/*nettoyage des access (grace au link > conomie de boucles)*/
		for(lp = current->access;lp;lp = lpp)
		{
			anUser *u = lp->value.a->user;
			lpp = lp->next;
			del_access(lp->value.a->user, current);
			write_anuser(u);
		}
		if(current->suspend) free(current->suspend);
		else cspart(current, raison);
		if(current->motd) free(current->motd);
		cswallops("%s", raison);
		mylog(LOG_CHANS, "[%s] %s\n", get_time(time(NULL)), raison);
		free(current);
		current = NULL;
		write_chanlist();
	}
	return;
}

void del_access(anUser *user, aChan *chan)
{
	anAccess *acces = user->accesshead, *last = NULL;
	aLink *lp = NULL, *lpp = NULL;

	for(;acces;last = acces, acces = acces->next)
		if(!strcasecmp(acces->chan, chan->chan))
		{
			if(last) last->next = acces->next;
			else user->accesshead = acces->next;
			break;
		}

	for(lp=chan->access;lp;lpp = lp, lp = lp->next)
		if(lp->value.a->user == user)
		{
			if(lpp) lpp->next = lp->next;
			else chan->access = lp->next;

			if(lp->value.a->info) free(lp->value.a->info);
			free(lp->value.a);
			free(lp); /* ne pas oublier de free() le link, -memory leak-*/
			return;
		}
	return;
}

void del_killinfo(aNick *nick, const char *against)
{
   aKill *current, *previous = NULL;

   current = killhead;
   while(current)/* del de la queue des kills si c'est celui qui pointe vers le nick */
   {			/*  et que nick->nick correspond au 'against', si NULL tout kill est retir*/
      if(current->nick == nick && (!against || !strcasecmp(nick->nick, against)))
      {
		  if(current == killhead) killhead = killhead->next;
		  else previous->next = current->next;
		  free(current);
		  return;
      }
      else
      {
         previous = current;
         current = current->next;
      }
   }
}

void del_abadnick(const char *chaine)
{
   struct badnicks *bn = badnhead, *previous = NULL;

   for(;bn && strcasecmp(chaine, bn->chaine);previous = bn, bn= bn->next);

   if(bn)
   {
      if(previous) previous->next = bn->next;
      else badnhead = bn->next;
      free(bn);
   }
}

void del_join(aNick *nick, const char *chan)
{
	aJoin *current, *previous = NULL;

	for(current = nick->joinhead;current;previous = current, current = current->next)
		if(!strcasecmp(chan, current->channel))
		{
			if(previous) previous->next = current->next;
			else nick->joinhead = current->next;

			if(current->chan) del_link(current->chan, nick); /* on del le link*/
			if(nick->user) /* on fout le lastseen comme il faut */
			{
				anAccess *acces = GetAccessIbyUserI(nick->user, chan);
				if(acces) acces->lastseen = time(NULL);
			}
			free(current);
			break;
		}

	return;
}

void del_link(aChan *chan, aNick *nick)
{
	register aLink *l, *lp = NULL;

	chan->users--;
	for(l=chan->members;l;lp = l, l = l->next)
		if(l->value.j->nick == nick)
		{
			if(lp) lp->next = l->next;
			else chan->members = l->next;
			free(l);
			return;
		}
}

void del_ban(aChan *chan, aBan *ban)
{
	if(chan->banhead == ban)
	{
		chan->banhead = ban->next;
		if(ban->next) chan->banhead->last = NULL;
	}
	else
	{
		ban->last->next = ban->next;
		if(ban->next) ban->next->last = ban->last;
	}
	free(ban->raison);
	free(ban);
	ban = NULL;
	return;
}

void del_regnick(anUser *user)
{
	anAccess *chan, *ct;
	int writechan = 0;
#ifdef USE_MEMOSERV
	aMemo *memo, *mem;
#endif

	File_UserDel(user->nick);

	if(user == userhead)
	{
		userhead = userhead->next;
		if(userhead->next) userhead->last = NULL;
	}
	else
	{
		if(user->last) user->last->next = user->next;
		if(user->next) user->next->last = user->last;
	}
	for(chan=user->accesshead;chan;chan=ct)
	{
		ct = chan->next;
		if(chan->level == OWNERLEVEL)
		{
			del_chan(chan->c, "delregnick de l'owner");
			writechan = 1;
		}
		else del_access(user, chan->c);
	}
#ifdef USE_MEMOSERV
	for(memo = user->memohead;memo;memo=mem)
	{
		mem = memo->next;
		free(memo);
	}
#endif
	if(user->suspend) free(user->suspend);
	free(user);
	user = NULL;
	write_userlist();
	if(writechan) write_chanlist();
}

void del_ignore(const char *host)
{
	struct ignore *i = ignorehead, *previous = NULL;

	for(;i;previous = i, i = i->next)
		if(!strcasecmp(host, i->host))
		{
			if(previous) previous->next = i->next;
			else ignorehead = i->next;
			free(i);
			return;
		}
}
