/* src/main.c - Fichier principal
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "init.h"

CVSINFO("$Id: main.c,v 1.36 2003/08/02 01:56:02 romexzf Exp $");

int premiere = 0;
time_t STS;
int lastresetparse = 0;
int lastresetlogs = 0;
int want_to_quit;
int lastcheckaccounts = 0;
struct userinfo *userhead = NULL;
struct nickinfo *nickhead = NULL;
struct badchans *badchead = NULL;
struct chaninfo *chanhead = NULL;
struct killinfo *killhead = NULL;
struct badnicks *badnhead = NULL;
struct ignore *ignorehead = NULL;
struct welcomeinfo *welcomehead = NULL;


/*
 *            FONCTIONS COMMANDES
 *            =-=-=-=-=-=-=-=-=-=
 *
 *    Il y a deux types de fonctions commandes.
 *    Les normales :
 *       parv[0] = pseudo du robot appel
 *       parv[1->parc] = arguments de la commandes
 *    Et les commandes salons (voir strict cs_commands) :
 *       parv[0] = nom de la commande appele
 *       parv[1] = salon appel
 *       parv[2->parv] = arguments de la commande (aprs le nom du salon)
 *    Toute les commandes doivent respecter ce schma.
 *    Lorsqu'une commande salon est tape sur un salon, et que le premier
 *    argument n'est pas un nom de salon, le nom du salon actuel est plac
 *    automatiquement comme premier argument.
 *
 *    Pour faciliter la lecture, et au cas o il pouvait y avoir plusieurs cas
 *    (ex: cmd op, if (parc == 0) cible = nick->nick else cible = parv[1]).
 *
 *    Chaque commande emploie une entte particulire :
 *    void commande(aNick *, aChan *, int, char **)
 *      aNick *nick = structure nickinfo de l'user qui a appel la fonction
 *      aChan *chaninfo = structure chaninfo du salon si la cmd est chan=1 (sinon chaninfo=NULL)
 *      int parc = nombre d'arguments *SANS COMPTER PARV[0]*. (si le dernier arg
 *                 est parv[5], alors parc = 5.)
 *      char **parv = tableau contenant les arguments (une ligne = un mot).
 *
 *    Outils utiles :
 *      parv2msgn(int parc, char **parv, int position, char buf, int size)
 *          permet de convertir une fin de tableau en un message normal.
 *          il faut spcifier le tableau et le nombre d'arguments contenu
 *          ainsi que la position  laquelle on va commencer  enregistrer
 *          le contenu dans la variable envoye en paramtre (taille max size).
 *      nick2numinfo(char nick), num2nickinfo(char num)
 *          renvoi la struct du pseudo spcifi dans l'un, et le pseudo d'une
 *          num spcifie dans l'autre.
 *     csntc(struct nickinfo *nick, char *message, ...)
 *          envoyer une notice avec le CS
 *    Pour plus d'outils, voir outils.c.
 */

void sig_handler (int c)
{
	want_to_quit = 1;
	signal(SIGINT, &sig_handler);
	signal(SIGTERM,&sig_handler);
}

void sig_reload (int c)
{
	signal(SIGHUP, &sig_reload);
}

void sig_alarm(int c)
{
	time_t current = time(NULL);
	bot.lasttime = current;
	bot.lastbytes = 0;
	check_kills();
	check_bans();
	check_ignores();
	if((lastcheckaccounts + 7200) < current)
	{
		check_accounts();
		check_chans();
#ifdef SQLLOG
#	ifdef DEBUG
		sql_query("DELETE FROM debuglog WHERE TS<=%lu;", lastcheckaccounts ? lastcheckaccounts : time(NULL) - 7200);
#	endif
#endif
		lastcheckaccounts = current;
	}
#ifndef SQLLOG
	if((lastresetparse + WAIT_DEL_PARSES) < current)
	{
		remove(LOG_PARSES);
		lastresetparse = current;
	}
#endif
	alarm(ALARMINTERVAL);
}

int main(int argc, char **argv)
{
	int r, restarted = 0, silent = 0, tmp = 0, opt, background = 1;
	FILE *fuptime;
	char buff[150] = {0};

	while((opt = getopt(argc, argv, "hn")) != EOF)
		switch(opt)
		{
			case 'n':
				background = 0;
				break;
			case 'h':
				silent = 1;
				break;
			default:
				printf("Syntax: %s [-hn]\n", argv[0]);
				exit(1);
				break;
		}

	chdir(BINDIR); /* on se place dans le rpertoire principal de scoderz */

	if((fuptime = fopen(SCODERZ_PID, "r")))
	{
		while(fgets(buff, 150, fuptime))
		{
			strip_newline(buff);
			sscanf(buff, "%d", &tmp);
		}
		fprintf(stderr, "SCoderZ est dj lanc sur le pid %d.\n", tmp);
		fprintf(stderr, "Si ce n'est pas le cas, supprimez le fichier '%s' et recommencez.\n", SCODERZ_PID);
		fclose(fuptime);
		exit(EXIT_FAILURE);
	}
	else bot.uptime = 0;

	if (load_config(FICHIER_CONF) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la configuration\n");
		exit(EXIT_FAILURE);
	}

	if((fuptime = fopen("uptime.tmp", "r")))
	{
		while(fgets(buff, 150, fuptime))
		{
			strip_newline(buff);
			sscanf(buff, "%lu %d %d", (unsigned long *) &bot.uptime, &lastresetparse, &lastresetlogs);
		}
		fclose(fuptime);
		remove("uptime.tmp");
		restarted = 1;
	}
	r = load_cmds();
	if(!silent) printf("Chargement des commandes IRC... OK (%d)\n", r);
	r = load_chans();
	if(!silent) printf("Base de donne Salon charge (%d)\n", r);
	r = load_users();
	if(!silent) printf("Base de donne User charge (%d)\n", r);
	r = load_badchans();
	if(!silent) printf("Chargement des bads chans... OK (%d)\n", r);
#ifdef USE_WELCOMESERV
	load_welcome();
#endif
	r = load_badnicks();
	if(!silent) printf("Chargement des bads nicks... OK (%d)\n", r);
#ifdef HAVE_VOTE
	load_votes();
#endif
	STS = time(NULL);
	signal(SIGPIPE, SIG_IGN);
	signal(SIGHUP, &sig_reload);
	signal(SIGINT, &sig_handler);
	signal(SIGTERM,&sig_handler);
	signal(SIGALRM, &sig_alarm);

	if(background)
	{
		r=fork();
		if (r == -1)
		{
			fprintf(stderr, "Impossible de se lancer en background.");
			exit(EXIT_FAILURE);
		}
	}

	if(!restarted && !silent && (!background || r)) printf("Services CoderZ %s  Inter System\n", SPVERSION);
	if(premiere && !silent)
	{
		printf("Lorsque les services seront sur votre rseau IRC,\n");
		printf("tapez : /%s register <username> <email> <pass>\n", cs.nick);
	}
	if(background && r > 1)
	{
		if(!restarted && !silent) printf("Lancement en background...\n");
		exit(0);
	}

#ifdef SQLLOG
	bot.sql_id = NULL;
	if(!sql_init())
	{
		printf("log:SQL connexion  %s:%d (user=%s base=%s) impossible...\n",
					bot.sql_ip, bot.sql_port, bot.sql_user, bot.sql_db);
		exit(EXIT_FAILURE);
	}
#endif

#ifdef TESTBOT
	w2c_initsocket();
#endif

    strcpy(bot.servnum, make_servnum(bot.num));
#ifdef N2K
    sprintf(cs.num, "%s%s", bot.servnum, "ACS");
#ifdef USE_WELCOMESERV
    sprintf(ws.num, "%s%s", bot.servnum, "AWS");
#endif

#else
    sprint(cs.num, "%s%s", bot.servnum, "CS");
#ifdef USE_WELCOMESERV
    sprintf(ws.num, "%s%s", bot.servnum, "WS");
#endif

#endif

	if(!(fuptime = fopen(SCODERZ_PID, "w")) && !silent) printf("Impossible d'crire le fichier PID...\n");
	else
	{
		fprintf(fuptime, "%d\n", getpid());
		fclose(fuptime);
	}

	while (!want_to_quit)
	{
		int irc_ip = resolve(bot.ip), sock = 0;

		if (!irc_ip || irc_ip == -1) sock = -1;
		else sock = init_bot(irc_ip, bot.port);

		bot.ipnum = irc_ip;

		if (sock < 0) putlog(LOG_PARSES, "Connection choue\n");
		else run_bot(sock);

		if (!want_to_quit)
		{
			aNick *nick = nickhead, *nt = NULL;
			aChan *chan = chanhead;
			putlog(LOG_PARSES, "Dconnect par le serveur\n");

			for(;nick;nick = nt)
			{
				nt = nick->next;
				del_nickinfo(nick);
			}

			for(;chan;chan = chan->next) DelCJoined(chan);
			close(sock);
			sleep(WAIT_CONNECT);
		}
	}
#ifdef SQLLOG
	if(bot.sql_id) mysql_close(bot.sql_id);
#endif
	putlog(LOG_PARSES, "Fermeture normale du programme\n");
	return 0;
}
