/* src/outils.c - Divers outils
 * Copyright (C) 2002-2003 Inter System
 *
 * contact: Progs_and_davidounet@Inter-System.Net
 *          Cesar@Inter-System.Net
 *          kouak@kouak.org
 * site web: http://coderz.inter-system.net
 *
 * Services pour serveur IRC. Support sur IrcProgs et IrCoderZ
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ctype.h>
#include <stdarg.h>
#include "main.h"
#ifdef USEBSD
#include <sys/types.h>
#include <netinet/in.h>
#endif
#include <arpa/inet.h>
#include <sys/socket.h>
#include <netdb.h>
#ifdef HAVE_CRYPTHOST
#include <crypt.h>
#endif

CVSINFO("$Id: outils.c,v 1.99 2003/08/05 22:14:11 romexzf Exp $");

int dataS = 0;
extern struct Link *mainhub;

extern struct badchans *badchead;
extern struct badnicks *badnhead;

extern struct cs_commands cs_cmds[];
extern const int ConfFlag;

static const char *months[12] = {
  "Janvier", "Fevrier", "Mars", "Avril", "Mai", "Juin", "Juillet", "Aot", "Septembre", "Octobre", "Novembre", "Decembre"
};

static const char *weekdays[7] = {
  "Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi"
};

struct Modes UMode[] = {
	{ 'i', N_INV },
	{ 'x', N_CRYPT },
	{ 'r', N_REG },
	{ 'o', N_OPER },
	{ 'k', N_SERVICE },
	{ 'a', N_ADM },
	{ 'p', N_GOD },
	{ 'f', N_FEMME },
	{ 'h', N_HOMME },
	{ 'g', N_DEBUG },
	{ 'w', N_WALLOPS },
	{ 'D', N_DIE },
	{ 'd', N_DEAF },
	{ 'H', N_SPOOF },
	{ 0, 0},

};

struct Modes CMode[] = {
	{ 'n', C_MMSG },
	{ 't', C_MTOPIC },
	{ 'i', C_MINV },
	{ 'l', C_MLIMIT },
	{ 'k', C_MKEY },
	{ 's', C_MSECRET },
	{ 'p', C_MPRIVATE },
	{ 'm', C_MMODERATE },
	{ 'c', C_MNOCTRL },
	{ 'C', C_MNOCTCP },
	{ 'O', C_MOPERONLY },
	{ 'r', C_MUSERONLY },
	{ 0, 0},
};

#ifdef HAVE_CRYPTHOST
char *hostprot(const char *host)
{
	u_int32_t sum, digest[4];
	static char newhost[HOSTLEN + 1];
	int ip = is_ip(host);
	char key1[HOSTLEN + 1], *key2 = strchr(host, '.'), *key = ip ? key2 : key1;

	Strncpy(key1, host, HOSTLEN);
	strtok(key1, ".");

	if(!host[0] % 2)
	{
		MD2_CTX context;
		MD2Init(&context);
		MD2Update(&context, (unsigned char *) key, strlen(key));
		MD2Update(&context, (unsigned char *) host, strlen(host));
		MD2Final((void *) digest, &context);
	}
	else
	{
		MD5_CTX context;
		MD5Init(&context);
		MD5Update(&context, (unsigned char *) key, strlen(key));
		MD5Update(&context, (unsigned char *) host, strlen(host));
		MD5Final((void *) digest, &context);
	}

	sum = digest[0] + digest[1] + digest[2] + digest[3];

	if (!ip) snprintf(newhost, sizeof newhost, "%X%s", sum, key2);
	else snprintf(newhost, sizeof newhost, "%s.%X", key1, sum);
	return newhost;
}
#endif

/* Note: La fonction chan_check a t reprise du CS5.0  IRCore*/
int chan_check(const char *chan, aNick *nick)
{
	struct badchans *badchan;
	const char *p = chan;

	if(strlen(chan) > REGCHANLEN)
		return csntc(nick, "\2Le nom du salon est trop long (limite %d)\2", REGCHANLEN);

	if(*p != '#' || strchr(chan, ','))
		return csntc(nick, "\2%s\2 n'est pas un nom de salon valide.", chan);

	if(!strcasecmp(bot.pchan, chan))
		return csntc(nick, "Salon non enregistrable.");

	while(*p)
		if(!isascii(*p++))
			return csntc(nick, "\2Le nom du salon ne doit contenir aucun caractre spcial");

	if((badchan = getbadchan(chan)))
		return csntc(nick, "Le salon \2%s\2 ne peut pas tre enregistr pour la raison suivante :\2 %s\2", chan, badchan->raison);

	return 1;
}

int is_num(const char *num)
{
   while(*num) if(!isdigit(*num++)) return 0;
   return 1;
}

int is_ip(const char *ip)
{
	char n = 0;
	unsigned int a = 0, b = 0, c = 0, d = 0;
	if(4 == sscanf(ip, "%u.%u.%u.%u%c\n", &a, &b, &c, &d, &n) && a < 256
			&& b < 256 && c < 256 && d < 256 && n == 0)
				return 1;
	return 0;
}

void strupr(char *buf)
{
	for(;*buf;buf++) *buf = toupper(*buf);
	*buf = '\0';
}

void strlwr(char *buf)
{
	for(;*buf;buf++) *buf = tolower(*buf);
	*buf = '\0';
}

void conv_char(char *chan, char *string)
{
	char val[10] = {0}, *ptr = chan;
	*string = '\0';

	while(*ptr)
	{
		if(!isalnum(*ptr)) sprintf(val, "%%%03o", *ptr);
		else sprintf(val, "%c", tolower(*ptr));
		strcat(string, val);
		ptr++;
	}
	return;
}

void strip_newline(char *string)
{
   register char *p = string;
   while(*p && *p != '\n' && *p != '\r') p++;
   *p = '\0';
}

int ChanLevelbyUserI(anUser *user, const char *chan)
{
	anAccess *acces = GetAccessIbyUserI(user,chan);
	return acces ? acces->level : 0;
}

aJoin *getjoininfo(aNick *nick, const char *chan)
{
	register aJoin *join = nick->joinhead;
	for(;join && strcasecmp(chan, join->channel);join=join->next);
	return join;
}

aChan *getchaninfo(const char *chan)
{
	register aChan *current = chanhead;
	for(;current && strcasecmp(chan,current->chan);current=current->next);
	return current;
}

anUser *getuserinfo(const char *nick)
{
	register anUser *current = userhead;
	for(;current && strcasecmp(nick,current->nick);current=current->next);
	return current;
}

aNick *getnickbynick(const char *nick)
{
	register aNick *current = nickhead;
	for(;current && strcasecmp(nick,current->nick); current = current->next);
	return current;
}

struct badnicks *getbadnick(const char *chan)
{
   struct badnicks *current = badnhead;
   for(;current && match(current->chaine, chan);current=current->next);
   return current;
}

int IsBadNick(const char *chan)
{
   struct badnicks *current;
   for(current = badnhead;current;current=current->next) if(!match(current->chaine, chan)) return 1;
   return 0;
}

struct badchans *getbadchan(const char *chan)
{
   struct badchans *current = badchead;
   for(;current && match(current->chaine, chan);current = current->next);
   return current;
}

char *IsAnOwner(anUser *user)
{
	anAccess *acces = user->accesshead;
	for(;acces && acces->level != OWNERLEVEL;acces = acces->next);
	return acces ? acces->chan : NULL;
}

aNick *num2nickinfo(const char *num)
{
   register aNick *nick = nickhead;

   for(;nick && strcmp(num, nick->numeric);nick = nick->next);
   return nick;
}

anAccess *GetAccessIbyUserI(anUser *user, const char *chan)
{
	anAccess *a=NULL;
	for(a = user->accesshead ;a && strcasecmp(a->chan,chan); a = a->next);
	return (a && !AWait(a)) ? a : NULL;
}

aNick *GetMemberIbyNum(aChan *chan, const char *num)
{
	register aLink *l = chan->members;
	for(;l && strcmp(l->value.j->nick->numeric, num);l = l->next);
	return l ? l->value.j->nick : NULL;
}

aNick *GetMemberIbyNick(aChan *chan, const char *nick)
{
	aLink *l = chan->members;
	for(;l && strcasecmp(l->value.j->nick->nick, nick);l = l->next);
	return l ? l->value.j->nick : NULL;
}

/*
 * getoption:
 * recherche dans un tableau char ** (de 'parc' entres)
 * l'expression "opt"  partir de l'lment 'deb'
 * -si exp_int est > 0:
 *  cela veut dire qu'on attend une valeur numerique apres le char 'opt'
 *  si c'est le cas, la fct renvoit la valeur, sinon 0
 * -si exp_int est < 0:
 *  cela veut dire que l'on recherche uniquement si 'opt' est dans parv,
 *  si c'est le cas, retourne 1
 * -si exp_int est = 0:
 *  cela veut dire que l'on attend une string apres le 'opt',
 *  si c'est le cas retourne l'index de cette string dans parv, sinon 0.
 *
 */

int getoption(const char *opt, char **parv, int parc, int deb, int exp_int)
{
	for(;deb <= parc;deb++)
		if(!strcasecmp(opt, parv[deb]))
		{
			if(exp_int < 0) return 1;
			if(++deb <= parc)
			{
				if(is_num(parv[deb]) && exp_int) return atoi(parv[deb]);
				else if(!exp_int) return deb;
			}
			break;
		}
	return 0;
}

struct Link *num2servinfo(const char *num)
{
	struct Link *link = mainhub;

	for(;link;link = link->next)
#ifdef N2K
		if(!strncmp(num, link->num, 2)) return link;
#else
		if(!strncmp(num, link->num, 1)) return link;
#endif
	return NULL;
}

struct Link *GetLinkIbyServ(const char *serv)
{
	struct Link *link = mainhub;
	for(;link && strcasecmp(link->serv, serv);link = link->next);
	return link;
}

void mylog(const char *filename, const char *format, ...)
{
   va_list arglist;
   FILE *fp = fopen(filename, "a");
   if(!fp) return;
   va_start(arglist, format);
   vfprintf(fp, format, arglist);
   va_end(arglist);
   fclose(fp);
   return;
}

void parv2msgn(unsigned int parc, char **parv, unsigned int base, char *buf, unsigned int size)
{
	unsigned int i = base, taillec = 0, towrite = 0;

	*buf = 0;
	size--;
	for(;i <= parc;i++)
	{
		if(!size) return;
		taillec = strlen(parv[i]);
		if(i > base)
		{
			strcat(buf, " ");
			size--;
		}
		if(size < taillec)
		{
			towrite = size;
			size = 0;
		}
		else
		{
			towrite = taillec;
			size -= taillec;
		}
		strncat(buf, parv[i], towrite);
	}
	return;
}

char *parv2msg(unsigned int parc, char **parv, unsigned int base, unsigned int size)
{
	unsigned int i = base, taillec = 0, towrite = 0;
	static char buf[500];
	buf[0] = '\0';

	size--;
	for(;i <= parc;i++)
	{
		if(!size) break;
		taillec = strlen(parv[i]);
		if(i > base)
		{
			strcat(buf, " ");
			size--;
		}
		if(size < taillec)
		{
			towrite = size;
			size = 0;
		}
		else
		{
			towrite = taillec;
			size -= taillec;
		}
		strncat(buf, parv[i], towrite);
	}
	return buf;
}

int ToggleFlag(int *flags, int flag)
{
	if(*flags & flag) *flags &= ~flag;
	else *flags |= flag;
	return *flags;
}

char *duration(int i)
{
	int mins = 0, hours = 0, days = 0;
	static char dur[200];
	char tmp2[30];
	dur[0] = '\0';

	if(i >= 60)
	{
		mins = i / 60;
		i -= 60 * mins;
	}
	if(mins >= 60)
	{
		hours = mins / 60;
		mins -= 60 * hours;
	}
	if(hours >= 24)
	{
		days = hours / 24;
		hours -= 24 * days;
	}

	if(days) {
	  sprintf(tmp2, "\002%d\2 jours", days);
	  strcat(dur, tmp2);
	}
	if(hours) {
	  if(days) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\2 heures", hours);
	  strcat(dur, tmp2);
	}
	if(mins) {
	  if(days||hours) strcat(dur, " ");
	  sprintf(tmp2, "\002%d\2 minutes", mins);
	  strcat(dur, tmp2);
	}
	if(!((mins || hours || days) && !i)) {
	  if(days||hours||mins) strcat(dur, " et ");
	  sprintf(tmp2, "\002%d\2 secondes", i);
	  strcat(dur, tmp2);
	}
	return dur;
}


time_t convert_duration(const char *duree)/* extrait une dure en secondes*/
{										/* d'un format XdXhXm.. - Cesar*/
	char *p = NULL, *p1, *s = p;
	time_t result = 0, t = 0;

	str_dup(&p, duree);
	p1 = p;

	while(*p1)
	{
		if(!isdigit(*p1) && !strchr("hHmYydjJD", *p1))
		{
			result = 0;
			break;
		}
		switch(*p1)
		{
			case 'H':
			case 'h':
						t = strtol(p, NULL, 10);
						result += 3600 * t;
						p = p1 + 1;
						break;
			case 'm':
						t = strtol(p, NULL, 10);
						result += 60 * t;
						p = p1 + 1;
						break;
			case 'Y':
			case 'y':
						t = strtol(p, NULL, 10);
						result += 31536000 * t;
						p = p1 + 1;
						break;
			case 'd':
			case 'j':
			case 'J':
			case 'D':
						t = strtol(p, NULL, 10);
						result += 3600 * 24 * t;
						p = p1 + 1;
						break;
		}
		p1++;
	}
	free(s);
	return result;
}

char *get_time(time_t ttime)
{
	register struct tm *lt;
	static char timet[70];

	lt = localtime(&ttime);

	snprintf(timet, 70, "%s %d %s %d %02d:%02d:%02d",
		weekdays[lt->tm_wday], lt->tm_mday, months[lt->tm_mon],
		1900 + lt->tm_year, lt->tm_hour, lt->tm_min, lt->tm_sec);

	return timet;
}

char *GetPrefix(aNick *nick)
{
	static char pref[12];
	pref[0] = '\0';

	if((nick->flag & N_OPER)) strcat(pref, "\2\0037*\2\003");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, "\00312^\003");
		else strcat(pref, "\0035!\003");
	}
	return pref;
}

char *GetChanPrefix(aNick *nick, aJoin *join, int colors)
{
	static char pref[14];
	pref[0] = '\0';

	if((nick->flag & N_OPER)) strcat(pref, colors ? "\2\0037*\2\003" : "*");
	if(nick->user)
	{
		if(IsAnAdmin(nick->user)) strcat(pref, colors ? "\00312^\003" : "^");
		else strcat(pref, colors ? "\0035!\003" : "!");
	}
	if(join->is_op) strcat(pref, "@");
	if(join->is_voice) strcat(pref, "+");

	return pref;
}

char *GetUserOptions(anUser *user)
{
	static char option[120];
	option[0] = '\0';

	if(UNopurge(user)) strcat(option, "NoPurge ");
	if(UNoMemo(user)) strcat(option, "NoMemo ");
	if(UFirst(user)) strcat(option, "Registering ");
	if(UOubli(user)) strcat(option, "Oubli ");
	if(UWantX(user)) strcat(option, "VirtualHost ");
	if(UWantDrop(user)) strcat(option, "Droping ");
	if(user->flag & U_ALREADYCHANGE) strcat(option, "UserChanged ");
	if(CantRegChan(user)) strcat(option, "CantRegChan ");
#ifdef USE_NICKSERV
	if(UPKill(user)) strcat(option, "Protection(Kill) ");
	else if(UPNick(user)) strcat(option, "Protection(ChNick) ");
#endif
	if(user->flag & U_PREJECT) strcat(option, "Accs(Rejet)");
	if(user->flag & U_POKACCESS) strcat(option, "Accs(Ask)");
	if(user->flag & U_PACCEPT) strcat(option, "Accs(Accepte)");

	if(option[0] == '\0') strcat(option, "Aucune");
	return option;
}

char *GetAccessOptions(anAccess *acces)
{
	static char options[100];
	options[0] = '\0';

	if(AOp(acces)) strcat(options, " autoop");
	if(AProtect(acces)) strcat(options, " protg");
	if(AVoice(acces)) strcat(options, " autovoice");
	if(ASuspend(acces)) strcat(options, " suspendu");
	if(options[0] == '\0') strcpy(options, " Aucune");
	return options;
}

int checkpass(const char *pass, anUser *user)
{
	return !strcmp(mkpass(pass), user->passwd);
}

void putlog(const char *fichier, const char *pattern, ...)
{
	char buf[512] = {0};
	va_list vl;

	if(!fichier || !*fichier) return;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);
	mylog(fichier, "[%s] %s", get_time(time(NULL)), buf);
}

void putserv(const char *pattern, ...)
{
	char buf[512] = {0};
	int len;

	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, sizeof buf, pattern, vl);
	va_end(vl);

	len = strlen(buf);
	send(bot.sock, buf, len, 0);
	dataS += len;

#ifdef DEBUG
#	ifdef N2K
	if(strncmp(&buf[3], TOKEN_PONG, 1) && strncmp(&buf[3], TOKEN_PING, 1))
#	else
	if(strncmp(&buf[2], TOKEN_PONG, 5) && strncmp(&buf[2], TOKEN_PING, 5))
#	endif
#	ifdef SQLLOG
	sql_query("INSERT INTO debuglog VALUES ('%s', UNIX_TIMESTAMP(), 'S');", buf);
#	else
    putlog(LOG_PARSES, "S - %s", buf);
#	endif
#endif
	return;
}

static const char table_dec[] = { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' };

int csntc(aNick *nick, register const char *pattern, ...)
{/* APACS N YYXXX :f*/
	char buf[512], t;
	int len = 15;
	va_list vl;
	register char *p = buf;

	va_start(vl, pattern);
	strcpy(p, cs.num);
#ifdef N2K
	*(p+5) = ' ';
	strcpy(p + 6, TOKEN_NOTICE);
	*(p+7) = ' ';
	strcpy(p + 8, nick->numeric);
	p += 13;
#else
	*(p+3) = ' ';
	strcpy(p + 4, TOKEN_NOTICE);
	*(p+10) = ' ';
	strcpy(p + 11, nick->numeric);
	p += 15;
	len += 2;
#endif
	*p++ = ' ';
	*p++ = ':'; /* prefix, cmd, cible, parser format*/

	while((t = *pattern++) && len < 509)/* %sa (t = %, *pattern = s)*/
	{
		if(t == '%')
		{
			t = *pattern++;/* on drop le formateur (t = s, *pattern = a)*/
			if(t == 's')
			{
				register char *tmps = va_arg(vl, char *);
				while(*tmps)/* copie de la string*/
				{
					*p++ = *tmps++;
					len++;
				}
				continue;
			}
			if(t == 'd')
			{
				int tmpi = va_arg(vl, int), pos = 31;
				char bufi[32];

				if(tmpi <= 0)
				{
					len++;
					if(!tmpi)
					{
						*p++ = '0';
						continue;
					}
					*p++ = '-';
					tmpi = -tmpi;
				}
				while(tmpi)/* on converti une int en base 10 en string*/
				{		/* criture dans l'ordre inverse 51 > '   1' > '  51'*/
					bufi[pos--] = table_dec[tmpi % 10];
					tmpi /= 10;
				}
				len += 31 - pos;
				while(pos < 31) *p++ = bufi[++pos];
				continue;
			}
			if(t == 'c')
			{
				*p++ = (char) va_arg(vl, int);
				len++;
				continue;
			}
			if(t != '%')
			{/* on sous traite (depuis le dbut..)  vsnprintf*/
				vsnprintf(p, sizeof(buf) - len, pattern - 2, vl);/* on remet le %format*/
				p = &buf[0] + (len = strlen(buf));
				break;
			}
		}
		*p++ = t;
		len++;
	}
	*p++ = '\r';
	*p++ = '\n';
	*p = '\0';
	len += 2;

	send(bot.sock, buf, len, 0);
	dataS += len;
	va_end(vl);
	return 0;
}

void putchan(const char *buf)
{
	time_t tmt = time(NULL);
	struct tm *ntime = localtime(&tmt);

	putserv("%s " TOKEN_PRIVMSG " %s :[%02d:%02d:%02d] %s\r\n", cs.num, bot.pchan, ntime->tm_hour,
		ntime->tm_min, ntime->tm_sec, buf);
	return;
}

void csmode(aChan *chan, const char *pattern, ...)
{
	char buf[512] = {0}, mode[50] = {0}, keyc[24] = {0}, limitc[24] = {0}, *tmp, *lp, *kp;
	int arg = 0, w = 1, cur = 0, limit = 0, key = 0;

	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);
	putserv("%s " TOKEN_MODE " %s %s\r\n", cs.num, chan->chan, buf);

	tmp = buf;
	kp = keyc;
	lp = limitc;
	while(*tmp && *tmp != ' ')/*parsage des modes & filtrage des +ovb & obtention du n des param de key/limite*/
	{
		switch(*tmp)
		{
			case 'o':
			case 'v':
			case 'b':
				arg++;
				break;
			case 'k':
				if(!key) key = ++arg; /* on garde que la premire key cas +kkk*/
				mode[cur++] = *tmp;
				break;
			case 'l':
				if(w && !limit) limit = ++arg;
				mode[cur++] = *tmp;
				break;
			case '-': w = 0;
			case '+': w = 1;
			default:
				mode[cur++] = *tmp;
				break;
		}
		tmp++;
	}
	tmp++; /* on saute l'espace*/
	arg = 1;
	while(*tmp)
	{
		if(*tmp == ' ')
		{
			arg++;
			tmp++;
		}
		else if(arg == limit) while(*tmp && *tmp != ' ') *lp++ = *tmp++; /* on lit le paramtre corrsspondant*/
		else if(arg == key) while(*tmp && *tmp != ' ') *kp++ = *tmp++;
		else tmp++;
	}
	/*++lp = '\0'; fin des char[]*/
	*++kp = 0;

	parse_cmode(chan, mode, limitc, key ? keyc : ((chan->key[0] != '\0') ? chan->key : NULL));
	return;
}

void cstopic(aChan *chan, const char *topic)
{
#ifdef N2K
	putserv("%s " TOKEN_TOPIC " %s %lu :%s\r\n", cs.num, chan->chan, time(NULL), topic);
#else
	putserv("%s " TOKEN_TOPIC " %s :%s\r\n", cs.num, chan->chan, topic);
#endif
	strcpy(chan->topic, topic);
}

void cskick(const char *chan, const char *vict, const char *pattern, ...)
{
	char buf[512] = {0};
	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 512, pattern, vl);
	va_end(vl);
	putserv("%s " TOKEN_KICK " %s %s :%s\r\n", cs.num, chan, vict, buf);
}

void cswallops(const char *pattern, ...)
{
	char buf[300] = {0};
	va_list vl;
	va_start(vl, pattern);
	vsnprintf(buf, 300, pattern, vl);
	va_end(vl);
	putserv("%s " TOKEN_WALLOPS " :%s\r\n", cs.num, buf);
}

void csjoin(aChan *chan)
{
	register aNick *n = nickhead;
	register aJoin *j;

	if(chan->members)/* on purge la nicklist du salon (si..) */
	{
		aLink *l, *lp = chan->members;
		for(;lp;lp = l)
		{
			l = lp->next;
			free(lp);
		}
		chan->users = 1;
		chan->members = NULL;
	}
	for(;n;n = n->next) /* chargement de la userlist HUGE load*/
		for(j = n->joinhead;j;j = j->next)
			if(!strcasecmp(chan->chan, j->channel))
			{	 /* on a trouv le bon chan hh */
				aLink *lp = calloc(1, sizeof(aLink));
				lp->value.j = j;
				lp->next = chan->members;
				chan->members = lp;
				chan->users++;
				j->chan = chan; /* on met  jour le pointeur */
				break;
			}

	if(!chan->members) putserv("%s " TOKEN_CREATE " %s %lu\r\n", cs.num, chan->chan, time(NULL));
	else/* auto op Create ou Join/+o*/
	{
		putserv("%s " TOKEN_JOIN " %s\r\n", cs.num, chan->chan);
		csmode(chan, "+o %s", cs.num);
	}
	if(chan->defmodes[0] != '\0') csmode(chan, "+%s", chan->defmodes[0] == '+' ? &chan->defmodes[1] : chan->defmodes);
	SetCJoined(chan);
}

void cspart(aChan *chan, const char *raison)
{
	aLink *l, *lp = chan->members;

	for(;lp;lp=l)/* nettoyage des chan members */
	{
		lp->value.j->chan = NULL; /* le salon n'est plus reg.. (sauf si part.)*/
		l = lp->next;
		free(lp);
	}
	chan->users = 1;
	chan->members = NULL;
	DelCJoined(chan);
	putserv("%s " TOKEN_PART " %s :%s\r\n", cs.num, chan->chan, raison);
}

void cs_account(aNick *nick, const char *account)
{
	if(!account && nick->user)
	{
		if(nick->user->n == nick) nick->user->n = NULL;
		nick->user = NULL;
	}
#ifdef N2K
	putserv("%s " TOKEN_ACCOUNT " %s %s\r\n", bot.servnum, nick->numeric, NONE(account));
#else
	putserv(":%s " TOKEN_ACCOUNT " %s %s\r\n", cs.nick, nick->numeric, NONE(account));
#endif
	return;
}

char *make_servnum(int c)
{
	static char res[3];
	const char base64[64] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789[]";
#ifndef N2K
	res[0] = base64[c & 63];
	res[1] = 0;
#else
	res[0] = base64[c / 64];
	res[1] = base64[c % 64];
	res[2] = '\0';
#endif

	return res;
}

int resolve(const char *host)
{
	struct hostent *he = gethostbyname(host);

	if(!he) return inet_addr(host);
	return *(int *)he->h_addr;
}

int check_protect(aNick *executant, aNick *execute, const char *chan)
{

   anAccess *access1, *access2;

   if(executant == execute) return 1;
   else if((execute->flag & N_GOD) && !(executant->flag & N_OPER) && !IsAnAdmin(executant->user))
   		return csntc(executant, "L'user \2%s\2 est +p", execute->nick);
   else if(!execute->user) return 1;
   else if(IsAnAdmin(executant->user) && executant->user->level > execute->user->level) return 1;
   /*l'executant est un admin de level suprieur a l'execut*/
   else if(IsAnAdmin(executant->user) && IsAnAdmin(execute->user)
   		&& executant->user->level <= execute->user->level && execute->user->level < 4)
   {
      csntc(executant, "L'user \2%s\2 est un Admin de niveau suprieur ou gal au votre.", execute->nick);
	  return 0; /*ce sont tous les 2 des admins mais executant est inferieur a execut*/
   }
   else if(!IsAnAdmin(executant->user) && IsAnAdmin(execute->user))/*executant n'est pas admin mais execute est admin*/
   		return csntc(executant, "L'user \2%s\2 est un Administrateur des Services.", execute->nick);

    /*optimisation: on check dabord l'acces de l'execut!*/
   if(!(access2 = GetAccessIbyUserI(execute->user, chan)) || ASuspend(access2) || !AProtect(access2)) return 1;

   /*on boucle les access de l'executant pour voir si on en trouve 1 sur le chan*/
   if(!(access1 = GetAccessIbyUserI(executant->user, chan)) || ASuspend(access1)) return 0;
   /*ne doit jamais arriver, voudrait dire que executant n'est pas un admin et n'a pas d'access sur le chan*/

   if(access1->level > access2->level) return 1; /*on verifie si executant a un level superieur a execute, ou si execute n'est pas protect;*/

   csntc(executant, "L'user \2%s\2 est protg ou a un access suprieur au votre sur \2%s\2.", execute->nick, chan);
   return 0; /*arrive pour tous les autres cas*/
}

void parse_umode(aNick *nick, char *p)
{
	int w = 1, i;

	if(!(nick && p)) return putlog(LOG_ERREURS, "ERREUR ARG: parse_umode\n");

	while(*p)
	{
		if(*p == '+') w = 1;
		else if(*p == '-') w = 0;
		else
			for (i = 0;UMode[i].mode;i++)
				if(*p == UMode[i].mode)
				{
					if(w) nick->flag |= UMode[i].value;
					else nick->flag &= ~UMode[i].value;
					break;
				}
		p++;
	}
	return;
}

char *GetModes(aNick *nick)
{
	static char mode[15];
	int i = 0, j = 0;
	for(;UMode[i].mode;i++) if((nick->flag & UMode[i].value)) mode[j++] = UMode[i].mode;
	mode[j] = '\0';
	return mode;
}

void parse_cmode(aChan *chan, char *p, const char *limit, const char *key)
{
	int w = 1, i;

	if(!chan) return putlog(LOG_ERREURS, "ERREUR ARG: parse_cmode\n");
	if(!p) return;
	while(*p)
	{
		if(*p == '+') w = 1;
		else if(*p == '-') w = 0;
		else for (i = 0;CMode[i].mode;i++)
				if(*p == CMode[i].mode)
				{
					if(w) chan->mode |= CMode[i].value;
					else chan->mode &= ~CMode[i].value;
					break;
				}
		p++;
	}
	if(chan->mode & C_MLIMIT)
	{
		if(limit && (i = atoi(limit))) chan->limit = i;
		else if(!chan->limit) chan->mode &= ~C_MLIMIT; /* dijiuu BugMaster! :p*/
	}
	else chan->limit = 0;
	if(chan->mode & C_MKEY)
	{
		if(key && key[0] != '\0') Strncpy(chan->key, key, sizeof chan->key - 1);
		else if(chan->key[0] == '\0') chan->mode &= ~C_MKEY;
	}
	else chan->key[0] = '\0';
	return;
}

char *GetCModes(aChan *chan)
{
	static char mode[40];
	int i = 0, j = 0;
	for(;CMode[i].mode;i++) if(chan->mode & CMode[i].value) mode[j++] = CMode[i].mode;
	mode[j] = '\0';
	if(chan->mode & C_MLIMIT) snprintf(mode + j, 15, " %d", chan->limit);
	return mode;
}

int IsValidNick(char *p)
{
	if(*p == '-' || isdigit(*p)) return 0;
	while(*p)
	{
		if(!isalnum(*p) && !strchr(SPECIAL_CHAR, *p)) return 0;
		p++;
	}
	return 1;
}

int IsValidMail(char *p)
{
	int fa = 0, fd = 0, ispoint = 0;
	if(strchr("-_.@", *p)) return 0;
	while(*p)
	{
		if(*p == '@')
		{
			if(!fa) fa = 1;
			else return 0;
		}
		if(fa && *p != '.') ispoint = 0;
		if(fa && *p == '.')
		{
			if(ispoint) return 0; /* le char d'avant etait un point, 2 points successifs dans l'url interdits */
			fd = 1;
			ispoint = 1;
		}
		if(!isalnum(*p) && *p != '.' && *p != '-' && *p != '_' && *p != '@') return 0;
		p++;
	}
	if(fa && fd) return 1;
	return 0;
}

int GetCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;cs_cmds[i].cmd[0] != '\0';i++) if(!strcasecmp(name, cs_cmds[i].realcmd)) return i;
	return -1;
}

int GetCurCmdIndexbyName(const char *name)
{
	unsigned int i = 0;

	for(;cs_cmds[i].cmd[0] != '\0';i++) if(!strcasecmp(name, cs_cmds[i].cmd)) return i;
	return -1;
}

char *RealCmd(const char *cmd)
{
	static char realcmd[15];
	int i = GetCmdIndexbyName(cmd);

	realcmd[0] = '\0';
	if(i >= 0) strcpy(realcmd, cs_cmds[i].cmd);
	return realcmd;
}

anUser *ParseNickOrUser(aNick *nick, char *arg)
{
	anUser *user = NULL;

	if(*arg == '%') /* recherche sur nick actif! */
	{
		aNick *n;
		if(*++arg == 0) csntc(nick, "Veuillez spcifier un nick aprs '%%' : COMMANDE %%nick-connect [args..]");
		else if(!(n = getnickbynick(arg))) csntc(nick, NOTCONNECTED, arg);
		else if(!n->user) csntc(nick, "%s n'est actuellement logu sur aucun UserName.", arg);
		else return n->user;
	}
	else if(!(user = getuserinfo(arg))) csntc(nick, NOSUCHUSER, arg);
	return user;
}

void File_UserDel(char *nick)
{
	char conv[130] = {0}, path[150];

	conv_char(nick, conv);
	snprintf(path, 150, DBDIR "/usr/%s", conv);
	remove(path);
	return;
}

int GetConf(const unsigned int flag)
{
	return (ConfFlag & flag) ? 1 : 0;
}

char *str_dup(char **to, const char *from)
{
	if(from && *from)
	{
		*to = realloc(*to, strlen(from) + 1);
		if(!to) putlog(LOG_ERREURS, "EXIT: strdup: no memory for 'to'(%p)=%s 'from'(%p)=%s\n",
			(void *) *to, *to, (void *) from, from);
		else strcpy(*to, from);
	}
	return *to;
}

char *Strncpy(char *to, const char *from, size_t n)/* copie from dans to. ne copie que n char*/
{							/* MAIS AJOUTE LE CHAR \0  la fin, from DOIT donc faire n+1 chars.*/
	const char *end = to + n;
	char *save = to;
	while(to < end && (*to++ = *from++));
	*to = 0;
	return save;
}
